/*
	Config section
*/

SMCParser g_hConfigParser;

enum EConfigState_t{
	State_None,
	State_Pets,
	State_InPet,
	State_PetSectionedInfo
}

EConfigState_t g_EConfigState;
PetInfo g_ParsePetInfo;
int g_iCurrentLine;
int g_iIgnoreLevel;

int g_iConfigCurrentSoundType = -1;

enum{
	//SoundInfo_Length, // not working for some reasons
	SoundInfo_Volume,
	SoundInfo_Cooldown,
}

float g_fConfigCurrentSoundInfo[3] = { 
											//0.0, 
											1.0, 
											0.0 
										};

#define CONFIG_DEFAULT_WALKBALE 	false
#define CONFIG_DEFAULT_MODEL 		"models/chicken/chicken.mdl"
#define CONFIG_DEFAULT_SKIN 		0
#define CONFIG_DEFAULT_SCALE		1.0
#define CONFIG_DEFAULT_COLOR_R	255
#define CONFIG_DEFAULT_COLOR_G	255
#define CONFIG_DEFAULT_COLOR_B	255
#define CONFIG_DEFAULT_COLOR_A	255
#define CONFIG_DEFAULT_GRAVITY	1.0
#define CONFIG_DEFAULT_IDLE		"idle01"
#define CONFIG_DEFAULT_RUN		"run01"
#define CONFIG_DEFAULT_FALL		"run01"

bool ParseConfig(){
	int iLen = g_hHandlesToClear.Length;
	if(iLen != -1){
		for(int i = 0; i < iLen; i++){
			delete view_as<StringMap>(g_hHandlesToClear.Get(i));
		}
		g_hHandlesToClear.Clear();
	}
	
	g_hMenu.RemoveAllItems();
	g_hMenu.AddItem("", "Disable");
	
	g_hPetsNames.Clear();
	g_hPetsList.Clear();
	
	g_fTeleportDistance = 1024.0;
	
	if(!g_hConfigParser){
		g_hConfigParser = new SMCParser();
		g_hConfigParser.OnEnterSection = Config_NewSection;
		g_hConfigParser.OnKeyValue = Config_KeyValue;
		g_hConfigParser.OnLeaveSection = Config_EndSection;
		g_hConfigParser.OnRawLine = Config_CurrentLine;
	}
	
	static char sPath[PLATFORM_MAX_PATH];
	if(!sPath[0]){
		BuildPath(Path_SM, sPath, sizeof(sPath), "data/vip/modules/pets.ini");
	}

	SMCError hErr = g_hConfigParser.ParseFile(sPath);
	if (hErr != SMCError_Okay){
		char sError[64];
		if (g_hConfigParser.GetErrorString(hErr, sError, sizeof(sError))){
			LogError("Failed to parse: '%s'. Line: %d. Error: %s", sPath, g_iCurrentLine, sError);
		}else{
			LogError("Failed to parse: '%s'. Line: %d. Unknown error.", g_iCurrentLine, sPath);
		}
		
		return false;
	}
	
	if(g_hPetsNames.Length == 0){
		g_hMenu.RemoveAllItems();
		LogMessage("Warning! VIP Pets config empty: '%s'.", sPath);
	}
	
	return true;
}

SMCResult Config_NewSection(SMCParser hParser, const char[] sName, bool bInQuotes){
	if (g_iIgnoreLevel){
		g_iIgnoreLevel++;
		return SMCParse_Continue;
	}
	
	switch(g_EConfigState){
		case State_None:{
			if(strcmp(sName, "Pets") == 0){
				g_EConfigState = State_Pets;
			}else{
				g_iIgnoreLevel++;
			}
		}
		case State_Pets:{
			strcopy(g_ParsePetInfo.Model, sizeof(g_ParsePetInfo.Model), CONFIG_DEFAULT_MODEL);
			
			g_ParsePetInfo.Walkable = CONFIG_DEFAULT_WALKBALE;
			g_ParsePetInfo.Skin = CONFIG_DEFAULT_SKIN;
			g_ParsePetInfo.Scale = CONFIG_DEFAULT_SCALE;
			g_ParsePetInfo.Color[0] = CONFIG_DEFAULT_COLOR_R;
			g_ParsePetInfo.Color[1] = CONFIG_DEFAULT_COLOR_G;
			g_ParsePetInfo.Color[2] = CONFIG_DEFAULT_COLOR_B;
			g_ParsePetInfo.Color[3] = CONFIG_DEFAULT_COLOR_A;
			g_ParsePetInfo.Gravity = CONFIG_DEFAULT_GRAVITY;
			
			g_ParsePetInfo.Position[0] = 0.0;
			g_ParsePetInfo.Position[1] = 0.0;
			g_ParsePetInfo.Position[2] = 0.0;
			
			g_ParsePetInfo.Rotation[0] = 0.0;
			g_ParsePetInfo.Rotation[1] = 0.0;
			g_ParsePetInfo.Rotation[2] = 0.0;
			g_ParsePetInfo.DefaultSoundsDisabled = false;
			
			g_ParsePetInfo.Sounds = new StringMap();
			g_ParsePetInfo.SoundsInfos = new StringMap();
			
			strcopy(g_ParsePetInfo.IdleAnimation, sizeof(g_ParsePetInfo.IdleAnimation), CONFIG_DEFAULT_IDLE);
			strcopy(g_ParsePetInfo.RunAnimation, sizeof(g_ParsePetInfo.RunAnimation), CONFIG_DEFAULT_RUN);
			strcopy(g_ParsePetInfo.FallAnimation, sizeof(g_ParsePetInfo.FallAnimation), CONFIG_DEFAULT_FALL);
		
			g_EConfigState = State_InPet;
			g_hMenu.AddItem("", sName);
			g_hPetsNames.PushString(sName);
			//strcopy(g_ParsePetInfo.Name, sName);
		}
		case State_InPet:{
		
			if(strcmp(sName, "idle_sound") == 0){
				g_iConfigCurrentSoundType = Sound_Idle;
			}else if(strcmp(sName, "panic_sound") == 0){
				g_iConfigCurrentSoundType = Sound_Panic;
			}else if(strcmp(sName, "fly_sound") == 0){
				g_iConfigCurrentSoundType = Sound_Fly;
			}else if(strcmp(sName, "wings_sound") == 0){
				g_iConfigCurrentSoundType = Sound_Wings;
			}
			
			g_EConfigState = State_PetSectionedInfo;
			
		}
		default:{
			g_iIgnoreLevel++;
		}
	}

	return SMCParse_Continue;
}

SMCResult Config_KeyValue(SMCParser hParser, const char[] sKey,  const char[] sValue, bool bKeyInQuotes, bool bValueInQuotes){
	if (g_iIgnoreLevel){
		return SMCParse_Continue;
	}
	
	switch(g_EConfigState){
		case State_Pets:{
			if(strcmp(sKey, "hide") == 0 && sValue[0] == '1'){
				g_bHidePetForEnemyTeam = true;
			}else if(strcmp(sKey, "walk_teleport_distance") == 0){
				g_fTeleportDistance = StringToFloat(sValue);
			}
		}
		case State_InPet:{
			
			if(strcmp(sKey, "model") == 0){
				strcopy(g_ParsePetInfo.Model, sizeof(g_ParsePetInfo.Model), sValue);
				PrecacheModel(sValue);
			}else if(strcmp(sKey, "position") == 0){
				ExplodeTo(sValue, g_ParsePetInfo.Position, 3, true);
			}else if(strcmp(sKey, "angles") == 0){
				ExplodeTo(sValue, g_ParsePetInfo.Rotation, 3, true);
			}else if(strcmp(sKey, "idle") == 0){
				strcopy(g_ParsePetInfo.IdleAnimation, sizeof(g_ParsePetInfo.IdleAnimation), sValue);
			}else if(strcmp(sKey, "run") == 0){
				strcopy(g_ParsePetInfo.RunAnimation, sizeof(g_ParsePetInfo.RunAnimation), sValue);
			}else if(strcmp(sKey, "fall") == 0){
				strcopy(g_ParsePetInfo.FallAnimation, sizeof(g_ParsePetInfo.FallAnimation), sValue);
			}else if(strcmp(sKey, "scale") == 0){
				g_ParsePetInfo.Scale = StringToFloat(sValue);
			}else if(strcmp(sKey, "skin") == 0){
				//strcopy(g_ParsePetInfo.Skin, sizeof(g_ParsePetInfo.Skin), sValue);
				g_ParsePetInfo.Skin = StringToInt(sValue);
			}else if(strcmp(sKey, "color") == 0){
				ExplodeTo(sValue, g_ParsePetInfo.Color, 4, false);
			}else if(strcmp(sKey, "walk") == 0){
				g_ParsePetInfo.Walkable = sValue[0] == '1';
			}else if(strcmp(sKey, "idle_sound") == 0){
				if(sValue[0]){
					PrecacheSound(sValue);
				}
				g_ParsePetInfo.Sounds.SetString("Idle", sValue);
			}else if(strcmp(sKey, "panic_sound") == 0){
				if(sValue[0]){
					PrecacheSound(sValue);
				}
				g_ParsePetInfo.Sounds.SetString("Panic", sValue);
			}else if(strcmp(sKey, "fly_sound") == 0){
				if(sValue[0]){
					PrecacheSound(sValue);
				}
				g_ParsePetInfo.Sounds.SetString("Fly", sValue);
			}else if(strcmp(sKey, "wings_sound") == 0){
				if(sValue[0]){
					PrecacheSound(sValue);
				}
				g_ParsePetInfo.Sounds.SetString("FlapWings", sValue);
			}else if(strcmp(sKey, "disable_default_sounds") == 0){
				g_ParsePetInfo.DefaultSoundsDisabled = sValue[0] == '1';
			}else if(strcmp(sKey, "gravity") == 0){
				g_ParsePetInfo.Gravity = StringToFloat(sValue);
			}
		}
		case State_PetSectionedInfo:{
			if(g_iConfigCurrentSoundType != -1){
				if(strcmp(sKey, "path") == 0){
					if(sValue[0]){
						PrecacheSound(sValue);
					}
					g_ParsePetInfo.Sounds.SetString(g_sSoundTypes[g_iConfigCurrentSoundType], sValue);
				//}else if(strcmp(sKey, "length") == 0){
				//	g_fConfigCurrentSoundInfo[SoundInfo_Length] = StringToFloat(sValue);
				}else if(strcmp(sKey, "volume") == 0){
					g_fConfigCurrentSoundInfo[SoundInfo_Volume] = StringToFloat(sValue);
				}else if(strcmp(sKey, "cooldown") == 0){
					g_fConfigCurrentSoundInfo[SoundInfo_Cooldown] = StringToFloat(sValue);
				}
			}
		}
	}
	
	return SMCParse_Continue;
}

void ExplodeTo(const char[] sBuffer, any[] aValues, int iLen, bool bFloat){
	
	char[][] sExplodedPosition = new char[iLen][16];
	ExplodeString(sBuffer, " ", sExplodedPosition, iLen, 16);
	
	for(int i = 0; i < iLen; i++){
		aValues[i] = bFloat ? view_as<any>(StringToFloat(sExplodedPosition[i])): view_as<any>(StringToInt(sExplodedPosition[i])); // tag mismatch fix ¯\_(ツ)_/¯
	}
}

SMCResult Config_EndSection(SMCParser hParser){
	if (g_iIgnoreLevel){
		g_iIgnoreLevel--;
		return SMCParse_Continue;
	}
	
	switch(g_EConfigState){
		case State_InPet:{
			
			if(!g_ParsePetInfo.Walkable || g_ParsePetInfo.Sounds.Size == 0){
				delete g_ParsePetInfo.Sounds;
				delete g_ParsePetInfo.SoundsInfos;
			}else{
				g_hHandlesToClear.Push(g_ParsePetInfo.Sounds);
				
				if(g_ParsePetInfo.SoundsInfos.Size == 0){
					delete g_ParsePetInfo.SoundsInfos;
				}else{
					g_hHandlesToClear.Push(g_ParsePetInfo.SoundsInfos);
				}
			}
		
			g_EConfigState = State_Pets;
			g_hPetsList.PushArray(g_ParsePetInfo, sizeof(PetInfo));
		}
		case State_Pets:{
			g_EConfigState = State_None;
		}
		case State_PetSectionedInfo:{
			
			
			// in case another sectioned info
			if(g_iConfigCurrentSoundType != -1){ 
				if(	
					//g_fConfigCurrentSoundInfo[SoundInfo_Length]		!= 	0.0		|| 
					g_fConfigCurrentSoundInfo[SoundInfo_Volume]		!= 	1.0		||
					g_fConfigCurrentSoundInfo[SoundInfo_Cooldown]	!= 	0.0
				){
					g_ParsePetInfo.SoundsInfos.SetArray(g_sSoundTypes[g_iConfigCurrentSoundType], g_fConfigCurrentSoundInfo, sizeof(g_fConfigCurrentSoundInfo));
				}

				//g_fConfigCurrentSoundInfo[SoundInfo_Length] 	= 	0.0;
				g_fConfigCurrentSoundInfo[SoundInfo_Volume] 	= 	1.0;
				g_fConfigCurrentSoundInfo[SoundInfo_Cooldown] 	= 	0.0;
				g_iConfigCurrentSoundType = -1;
			}
			
			g_EConfigState = State_InPet;
		}
	}

	//g_EConfigState--;
	return SMCParse_Continue;
}

SMCResult Config_CurrentLine(SMCParser hParser, const char[] sLine, int iLineNum){
	g_iCurrentLine = iLineNum;
	return SMCParse_Continue;
}